unit WinMemsCalibrateUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, ComCtrls, Proc, Mask;

type
  TWinMemsCalibrate = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    Label2: TLabel;
    Gyro_bias_X: TEdit;
    Label3: TLabel;
    Gyro_bias_Y: TEdit;
    Label4: TLabel;
    Gyro_bias_Z: TEdit;
    Label5: TLabel;
    Mag_bias_X: TEdit;
    Mag_bias_Y: TEdit;
    Label6: TLabel;
    Label7: TLabel;
    Mag_bias_Z: TEdit;
    x_max: TLabel;
    x_min: TLabel;
    y_min: TLabel;
    y_max: TLabel;
    z_min: TLabel;
    z_max: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    SpeedButton4: TSpeedButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure SpeedButton4Click(Sender: TObject);
  private
    InitValues: TStringList;
    procedure ApplyChanges;
    function FindControl(cn: string): TControl;
    function GetControlValue(tc: TControl): string;
    procedure LoadVariable(k, v: string);
    procedure SetControlValue(tc: TControl; v: string);
    { Private declarations }
  public
    { Public declarations }
    Mag_cal_started: boolean;
  end;

var
  WinMemsCalibrate: TWinMemsCalibrate;

implementation

uses MAIN;

{$R *.dfm}

procedure TWinMemsCalibrate.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
end;

procedure TWinMemsCalibrate.FormDestroy(Sender: TObject);
begin
  if Mag_cal_started then MainForm.AddToOutput('cal mag_cancel'+#13#10);

  InitValues.Free;
  WinMemsCalibrate:=nil;
end;

procedure TWinMemsCalibrate.FormCreate(Sender: TObject);
var
  s,k,v: string;
begin
  InitValues:= TStringList.Create;
  MainForm.ClearInput;
  MainForm.AddToOutput('cat mag'+#13#10);
  while true do begin
    s:=MainForm.GetNextInputString;
    if s<>'' then begin
      if s='=>' then break;
      if ExtractKeyValue(s,k,v) then begin
        LoadVariable(k,v);
      end else continue;
    end;
  end;

  MainForm.AddToOutput('cat gyro'+#13#10);
  while true do begin
    s:=MainForm.GetNextInputString;
    if s<>'' then begin
      if s='=>' then break;
      if ExtractKeyValue(s,k,v) then begin
        LoadVariable(k,v);
      end else continue;
    end;
  end;

end;


procedure TWinMemsCalibrate.ApplyChanges;
var
  i: integer;
  k,v: string;
   tc,tc_x,tc_y: TControl;
   ControlName: string;
   new_v,x,y,new_xv,new_yv: string;
begin
  for i:=0 to InitValues.Count-1 do begin
    k:=InitValues.Names[i];
    v:=InitValues.ValueFromIndex[i];
    tc:=FindControl(k);
    if(tc<>nil) then begin
      new_v:=GetControlValue(tc);
      if (new_v<>'') and (new_v <> v) then begin
         MainForm.AddToOutput('set '+k+' '+ new_v +#13#10);
         InitValues.Values[k]:=new_v;
      end;
    end;
  end;
end;

function TWinMemsCalibrate.FindControl(cn: string): TControl;
begin
  Result:=GroupBox1.FindChildControl(cn);
  if Result = nil then begin
    Result:=GroupBox2.FindChildControl(cn);
    if Result = nil then exit;
  end;
end;

function TWinMemsCalibrate.GetControlValue(tc: TControl): string;
begin
  Result:='';
  if tc.ClassName='TEdit' then begin
     Result:=(tc as TEdit).Text;
     Result:=StringReplace(Result,',','.',[rfReplaceAll]);
  end;
  if tc.ClassName='TComboBox' then begin
     Result:=IntToStr((tc as TComboBox).ItemIndex);
  end;
  if tc.ClassName='TUpDown' then begin
     Result:=IntToStr((tc as TUpDown).Position)
  end;
  if tc.ClassName='TCheckBox' then begin
     if tc.Tag = 0 then begin
       Result:='0';
       if (tc as TCheckBox).Checked then Result:='1';
     end else begin
       Result:='1';
       if (tc as TCheckBox).Checked then Result:='-1';
     end;
  end;
end;

procedure TWinMemsCalibrate.LoadVariable(k, v: string);
var
   tc,tc_x,tc_y: TControl;
   x,y: string;
   xv,yv: string;
   i: integer;
begin
  tc:=FindControl(k);
  if(tc<>nil) then begin
    SetControlValue(tc,v);
    InitValues.Values[k]:=v;
  end;
end;

procedure TWinMemsCalibrate.SetControlValue(tc: TControl; v: string);
begin
  if tc.ClassName='TEdit' then begin
     (tc as TEdit).Text := v;
  end;
  if tc.ClassName='TComboBox' then begin
     (tc as TComboBox).ItemIndex := StrToInt(v);
  end;
  if tc.ClassName='TUpDown' then begin
     (tc as TUpDown).Position := StrToInt(v);
  end;
  if tc.ClassName='TCheckBox' then begin
     if tc.Tag = 0 then begin
       if v='0' then  (tc as TCheckBox).Checked:=false else (tc as TCheckBox).Checked:=true;
     end else begin
       if v='1' then  (tc as TCheckBox).Checked:=false else (tc as TCheckBox).Checked:=true;
     end;
  end;
end;


procedure TWinMemsCalibrate.SpeedButton1Click(Sender: TObject);
var
   s,k,v: string;
begin
  Label8.Visible:=true;
  MainForm.AddToOutput('cal gyro'+#13#10);
  Application.ProcessMessages;
  Sleep(1000);
  MainForm.ClearInput;
  while true do begin
    Application.ProcessMessages;
    s:=MainForm.GetNextInputString;
    if s<>'' then begin
      if s='=>' then break;
      if ExtractKeyValue(s,k,v) then begin
        LoadVariable(k,v);
      end else continue;
    end;
  end;
  Label8.Visible:=false;

end;

procedure TWinMemsCalibrate.SpeedButton2Click(Sender: TObject);
var
    s,k,v: string;
    i: integer;
begin
  Mag_cal_started:=true;

  Label9.Visible:=true;
  x_max.Visible:=true;
  x_min.Visible:=true;
  y_max.Visible:=true;
  y_min.Visible:=true;
  z_max.Visible:=true;
  z_min.Visible:=true;

  MainForm.AddToOutput('cal mag'+#13#10);
  Application.ProcessMessages;
  Sleep(1000);
  MainForm.ClearInput;
  while Mag_cal_started do begin
    Application.ProcessMessages;
    s:=MainForm.GetNextInputString;
    if s<>'' then begin
      if s='=>' then break;

      i:=Pos('x_max=',s);
      if i>0 then begin
        Delete(s,1,i+5);
        i:=pos(' ',s);
        x_max.Caption:=Copy(s,1,i);
      end;


      i:=Pos('x_min=',s);
      if i>0 then begin
        Delete(s,1,i+5);
        i:=pos(' ',s);
        x_min.Caption:=Copy(s,1,i);
      end;

      i:=Pos('y_max=',s);
      if i>0 then begin
        Delete(s,1,i+5);
        i:=pos(' ',s);
        y_max.Caption:=Copy(s,1,i);
      end;


      i:=Pos('y_min=',s);
      if i>0 then begin
        Delete(s,1,i+5);
        i:=pos(' ',s);
        y_min.Caption:=Copy(s,1,i);
      end;

      i:=Pos('z_max=',s);
      if i>0 then begin
        Delete(s,1,i+5);
        i:=pos(' ',s);
        z_max.Caption:=Copy(s,1,i);
      end;


      i:=Pos('z_min=',s);
      if i>0 then begin
        Delete(s,1,i+5);
        z_min.Caption:=Copy(s,1,length(s));
      end;


    end;
  end;

  MainForm.ClearInput;
  MainForm.AddToOutput('cal mag_save'+#13#10);
  Application.ProcessMessages;
  while true do begin
    Application.ProcessMessages;
    s:=MainForm.GetNextInputString;
    if s<>'' then begin
      if s='=>' then break;
      if ExtractKeyValue(s,k,v) then begin
        if k<>'x_max' then LoadVariable(k,v);
      end else continue;
    end;
  end;


  Label9.Visible:=false;
  x_max.Visible:=false;
  x_min.Visible:=false;
  y_max.Visible:=false;
  y_min.Visible:=false;
  z_max.Visible:=false;
  z_min.Visible:=false;

end;

procedure TWinMemsCalibrate.SpeedButton3Click(Sender: TObject);
begin
  Mag_cal_started:=false;
end;

procedure TWinMemsCalibrate.BitBtn2Click(Sender: TObject);
begin
  ApplyChanges;
end;

procedure TWinMemsCalibrate.BitBtn1Click(Sender: TObject);
begin
  ApplyChanges;
  Close;
end;

procedure TWinMemsCalibrate.SpeedButton4Click(Sender: TObject);
begin
  MainForm.ClearInput;
  MainForm.AddToOutput('save ref'+#13#10);
  ShowMessage('');
end;

end.
